diff --git tests/phpunit/tests/multisite/networkQuery.php tests/phpunit/tests/multisite/networkQuery.php
index 0b48fdc8d5..8953012386 100644
--- tests/phpunit/tests/multisite/networkQuery.php
+++ tests/phpunit/tests/multisite/networkQuery.php
@@ -525,6 +525,7 @@ if ( is_multisite() ) :
 
 		/**
 		 * @ticket 45749
+		 * @ticket 47599
 		 */
 		public function test_networks_pre_query_filter_should_bypass_database_query() {
 			global $wpdb;
@@ -534,11 +535,7 @@ if ( is_multisite() ) :
 			$num_queries = $wpdb->num_queries;
 
 			$q       = new WP_Network_Query();
-			$results = $q->query(
-				array(
-					'fields' => 'ids',
-				)
-			);
+			$results = $q->query( array() );
 
 			remove_filter( 'networks_pre_query', array( __CLASS__, 'filter_networks_pre_query' ), 10, 2 );
 
diff --git tests/phpunit/tests/multisite/siteQuery.php tests/phpunit/tests/multisite/siteQuery.php
index c17f977932..e368642675 100644
--- tests/phpunit/tests/multisite/siteQuery.php
+++ tests/phpunit/tests/multisite/siteQuery.php
@@ -914,6 +914,7 @@ if ( is_multisite() ) :
 
 		/**
 		 * @ticket 45749
+		 * @ticket 47599
 		 */
 		public function test_sites_pre_query_filter_should_bypass_database_query() {
 			global $wpdb;
@@ -923,11 +924,7 @@ if ( is_multisite() ) :
 			$num_queries = $wpdb->num_queries;
 
 			$q       = new WP_Site_Query();
-			$results = $q->query(
-				array(
-					'fields' => 'ids',
-				)
-			);
+			$results = $q->query( array() );
 
 			remove_filter( 'sites_pre_query', array( __CLASS__, 'filter_sites_pre_query' ), 10, 2 );
 

diff --git src/wp-includes/class-wp-network-query.php src/wp-includes/class-wp-network-query.php
index 9a0f97a912..d1a437aafc 100644
--- src/wp-includes/class-wp-network-query.php
+++ src/wp-includes/class-wp-network-query.php
@@ -197,51 +197,56 @@ class WP_Network_Query {
 		 */
 		do_action_ref_array( 'pre_get_networks', array( &$this ) );
 
-		$network_ids = null;
+		$network_data = null;
 
 		/**
 		 * Filter the sites array before the query takes place.
 		 *
-		 * Return a non-null value to bypass WordPress's default site queries.
+		 * Return a non-null value to bypass WordPress's default network queries.
 		 *
+		 * The expected return type from this filter depends on the value passed in the request query_vars:
+		 * When $this->query_vars['count'] is set, the filter should return the network count as an int.
+		 * When `'ids' === $this->query_vars['fields']`, the filter should return an array of network ids.
+		 * Otherwise the filter should return an array of WP_Network objects.
 		 *
 		 * @since 5.2.0
 		 *
 		 * @param array|null       $site_ids Return an array of site data to short-circuit WP's site query,
 		 *                                   or null to allow WP to run its normal queries.
 		 * @param WP_Network_Query $this     The WP_Network_Query instance, passed by reference.
 		 */
-		$network_ids = apply_filters_ref_array( 'networks_pre_query', array( $network_ids, &$this ) );
+		$network_data = apply_filters_ref_array( 'networks_pre_query', array( $network_data, &$this ) );
 
-		if ( null === $network_ids ) {
-
-			// $args can include anything. Only use the args defined in the query_var_defaults to compute the key.
-			$_args = wp_array_slice_assoc( $this->query_vars, array_keys( $this->query_var_defaults ) );
+		if ( null !== $network_data ) {
+			return $network_data;
+		}
 
-			// Ignore the $fields argument as the queried result will be the same regardless.
-			unset( $_args['fields'] );
+		// $args can include anything. Only use the args defined in the query_var_defaults to compute the key.
+		$_args = wp_array_slice_assoc( $this->query_vars, array_keys( $this->query_var_defaults ) );
 
-			$key          = md5( serialize( $_args ) );
-			$last_changed = wp_cache_get_last_changed( 'networks' );
+		// Ignore the $fields argument as the queried result will be the same regardless.
+		unset( $_args['fields'] );
 
-			$cache_key   = "get_network_ids:$key:$last_changed";
-			$cache_value = wp_cache_get( $cache_key, 'networks' );
+		$key          = md5( serialize( $_args ) );
+		$last_changed = wp_cache_get_last_changed( 'networks' );
 
-			if ( false === $cache_value ) {
-				$network_ids = $this->get_network_ids();
-				if ( $network_ids ) {
-					$this->set_found_networks();
-				}
+		$cache_key   = "get_network_ids:$key:$last_changed";
+		$cache_value = wp_cache_get( $cache_key, 'networks' );
 
-				$cache_value = array(
-					'network_ids'    => $network_ids,
-					'found_networks' => $this->found_networks,
-				);
-				wp_cache_add( $cache_key, $cache_value, 'networks' );
-			} else {
-				$network_ids          = $cache_value['network_ids'];
-				$this->found_networks = $cache_value['found_networks'];
+		if ( false === $cache_value ) {
+			$network_ids = $this->get_network_ids();
+			if ( $network_ids ) {
+				$this->set_found_networks();
 			}
+
+			$cache_value = array(
+				'network_ids'    => $network_ids,
+				'found_networks' => $this->found_networks,
+			);
+			wp_cache_add( $cache_key, $cache_value, 'networks' );
+		} else {
+			$network_ids          = $cache_value['network_ids'];
+			$this->found_networks = $cache_value['found_networks'];
 		}
 
 		if ( $this->found_networks && $this->query_vars['number'] ) {
@@ -197,51 +197,56 @@ class WP_Network_Query {
 
 		/**
-		 * Filter the sites array before the query takes place.
+		 * Filter the network data before the query takes place.
 		 *
@@ -197,51 +197,56 @@ class WP_Network_Query {
 		 * @since 5.2.0
 		 *
-		 * @param array|null       $site_ids Return an array of site data to short-circuit WP's site query,
-		 *                                   or null to allow WP to run its normal queries.
-		 * @param WP_Network_Query $this     The WP_Network_Query instance, passed by reference.
+		 * @param array|null       $network_data Return an array of network data (ints, objects or a count) to short-circuit WP's network query,
+		 *                                       or null to allow WP to run its normal queries.
+		 * @param WP_Network_Query $this         The WP_Network_Query instance, passed by reference.
 		 */
diff --git src/wp-includes/class-wp-site-query.php src/wp-includes/class-wp-site-query.php
index ae91c8269e..5925c9239e 100644
--- src/wp-includes/class-wp-site-query.php
+++ src/wp-includes/class-wp-site-query.php
@@ -288,51 +288,56 @@ class WP_Site_Query {
 			$this->meta_query_clauses = $this->meta_query->get_sql( 'blog', $wpdb->blogs, 'blog_id', $this );
 		}
 
-		$site_ids = null;
+		$site_data = null;
 
 		/**
 		 * Filter the sites array before the query takes place.
 		 *
 		 * Return a non-null value to bypass WordPress's default site queries.
 		 *
+		 * The expected return type from this filter depends on the value passed in the request query_vars:
+		 * When $this->query_vars['count'] is set, the filter should return the site count as an int.
+		 * When `'ids' == $this->query_vars['fields']`, the filter should return an array of site ids.
+		 * Otherwise the filter should return an array of WP_Site objects.
 		 *
 		 * @since 5.2.0
 		 *
 		 * @param array|null    $site_ids Return an array of site data to short-circuit WP's site query,
 		 *                                or null to allow WP to run its normal queries.
 		 * @param WP_Site_Query $this The WP_Site_Query instance, passed by reference.
 		 */
-		$site_ids = apply_filters_ref_array( 'sites_pre_query', array( $site_ids, &$this ) );
+		$site_data = apply_filters_ref_array( 'sites_pre_query', array( $site_data, &$this ) );
 
-		if ( null === $site_ids ) {
-
-			// $args can include anything. Only use the args defined in the query_var_defaults to compute the key.
-			$_args = wp_array_slice_assoc( $this->query_vars, array_keys( $this->query_var_defaults ) );
+		if ( null !== $site_data ) {
+			return $site_data;
+		}
 
-			// Ignore the $fields argument as the queried result will be the same regardless.
-			unset( $_args['fields'] );
+		// $args can include anything. Only use the args defined in the query_var_defaults to compute the key.
+		$_args = wp_array_slice_assoc( $this->query_vars, array_keys( $this->query_var_defaults ) );
 
-			$key          = md5( serialize( $_args ) );
-			$last_changed = wp_cache_get_last_changed( 'sites' );
+		// Ignore the $fields argument as the queried result will be the same regardless.
+		unset( $_args['fields'] );
 
-			$cache_key   = "get_sites:$key:$last_changed";
-			$cache_value = wp_cache_get( $cache_key, 'sites' );
+		$key          = md5( serialize( $_args ) );
+		$last_changed = wp_cache_get_last_changed( 'sites' );
 
-			if ( false === $cache_value ) {
-				$site_ids = $this->get_site_ids();
-				if ( $site_ids ) {
-					$this->set_found_sites();
-				}
+		$cache_key   = "get_sites:$key:$last_changed";
+		$cache_value = wp_cache_get( $cache_key, 'sites' );
 
-				$cache_value = array(
-					'site_ids'    => $site_ids,
-					'found_sites' => $this->found_sites,
-				);
-				wp_cache_add( $cache_key, $cache_value, 'sites' );
-			} else {
-				$site_ids          = $cache_value['site_ids'];
-				$this->found_sites = $cache_value['found_sites'];
+		if ( false === $cache_value ) {
+			$site_ids = $this->get_site_ids();
+			if ( $site_ids ) {
+				$this->set_found_sites();
 			}
+
+			$cache_value = array(
+				'site_ids'    => $site_ids,
+				'found_sites' => $this->found_sites,
+			);
+			wp_cache_add( $cache_key, $cache_value, 'sites' );
+		} else {
+			$site_ids          = $cache_value['site_ids'];
+			$this->found_sites = $cache_value['found_sites'];
 		}
 
 		if ( $this->found_sites && $this->query_vars['number'] ) {
@@ -288,51 +288,56 @@ class WP_Site_Query {
 
 		/**
-		 * Filter the sites array before the query takes place.
+		 * Filter the site data before the get_sites query takes place.
 		 *
 		 * Return a non-null value to bypass WordPress's default site queries.
@@ -288,51 +288,56 @@ class WP_Site_Query {
 		 * @since 5.2.0
 		 *
-		 * @param array|null    $site_ids Return an array of site data to short-circuit WP's site query,
-		 *                                or null to allow WP to run its normal queries.
-		 * @param WP_Site_Query $this The WP_Site_Query instance, passed by reference.
+		 * @param array|int|null $site_data Return an array of site data (ints, objects or a count) to short-circuit
+		 *                                  WordPress's site query, or null to run the normal queries.
+		 * @param WP_Site_Query  $this      The WP_Site_Query instance, passed by reference.
 		 */